const plugin = editor => {

  const settings = editor.settings.colorPalette;
  const modules = settings && settings.module;
  const defaultColor = settings && settings.default_color;
  const $ = editor.$(window.parent.document);

  editor.addButton('colorPalette', {
    icon: 'forecolor',
    type: 'panelbutton',
    tooltip: '글자색',
    popoverAlign: ['bc-tl'],
    panel: {
      role: 'dialog',
      ariaRemember: true,
      html: generatePalette(settings),
    },
    onshow() {

      this.selectedTd = getSelectedTd();
      this.panel.$el[0].focus();

      if(this.isInitiated) {
        return;
      }

      // 이 밑으로는 매번 실행될 필요가 없다.
      this.isInitiated = true;
      if(modules.includes('preset')) {
        $.find('.colorPalette-preset-td').on('click', clickEventHandler.bind(this));
      }

      if(modules.includes('palette')) {
        $.find('.colorPalette-palette-td').on('click', clickEventHandler.bind(this));
      }

      if(modules.includes('code')) { // 일단 hex값으로 입력받는 부분은 스펙에서 제외.
        const input = document.getElementById('colorPalette-code-input');
        const displayBox = document.getElementById('colorPalette-code-display');
        const submitBtn = document.getElementById('colorPalette-code-submit');

        input.addEventListener("keyup", (e) => {
          if(e.keyCode === 13 && !submitBtn.disabled) {
            const color = (input.value.length === 3 || input.value.length === 6) ? '#' + input.value : input.value;
            applyFormat('forecolor', color);
            this.hidePanel();
          }
          if(isValidHexCode(input.value)) {
            let value = input.value;
            if(value.length === 3 || value.length === 6) {
              value = '#' + value;
            }
            displayBox.setAttribute('style', `background-color: ${value}`);
          }
          submitBtn.disabled = !submitBtn.disabled;
        });

        submitBtn.addEventListener('click', () => {
          const color = (input.value.length === 6 || input.value.length === 3) ? '#' + input.value : input.value;
          applyFormat('forecolor', color);
          this.hidePanel();
        })
      }
    },
    onhide() {
      editor.focus();
      this.selectedTd.classList.remove('colorPalette-selected');
    }
  })

  const clickEventHandler = function (e) {
    this.hidePanel();
    editor.focus();
    const color = e.currentTarget.getAttribute('data-mce-color');
    const sel = editor.selection.getSel();
    if (!sel.isCollapsed) {
      if(color === defaultColor) {
        editor.undoManager.transact(function() {
          removeFormat('forecolor');
        })
      }
      else {
        editor.undoManager.transact(function() {
          applyFormat('forecolor', color);
        })
      }
    }
    else {
      editor.undoManager.ignore(function() {
        editor.execCommand('mceInsertContent', false, `<span style='color: ${color};'</span>`);
      })
      applyFormat('forecolor', color);
    }
  }

  const applyFormat = function (format, value) {
    editor.focus();
    editor.formatter.apply(format, { value });
    editor.nodeChanged();
  };

  const removeFormat = function (format) {
    editor.focus();
    editor.formatter.remove(format, { value: null }, null, true);
    editor.nodeChanged();
  };

  const getSelectedTd = function () {
    const selectedNode = editor.selection.getSel().anchorNode;
    const parentSpans = editor.dom.getParents(selectedNode, 'span');
    let parentSpan;
    for(let span of parentSpans) {
      if(span.id !== '_mce_caret' && span.style && span.style.color) {
        parentSpan = span;
        break;
      }
    }
    const selectedHex = parentSpan ? convertToHexCode(parentSpan.style.color) : defaultColor;
    const selectedTd = $.find(`#colorPalette-preset-id-${settings.color_map_preset.indexOf(selectedHex)}`)[0];
    selectedTd.classList.add("colorPalette-selected");
    return selectedTd;
  }

  const convertToHexCode = colorString => {
    if(!colorString) {
      return defaultColor;
    }

    const m = /rgb\((\d*), (\d*), (\d*)\)/gi.exec(colorString)

    return '#' + m.slice(1).reduce((res, cur) => {
      let parsedHex = parseInt(cur).toString(16);
      if(parsedHex.length % 2) {
        parsedHex = '0' + parsedHex;
      }
      return res + parsedHex;
    }, "").toUpperCase();
  }
}

const isValidHexCode = (code) => {
  const len = code.length;
  if(len === 3 || len === 6) {
    for(let i = 0; i < len; i++) {
      if(!((0 <= code[i] && code[i] <= 9) || ('A' <= code[i] && code[i] <= 'F'))) {
        return false;
      }
    }
    return true;
  }
  else if(len === 4 || len === 7) {
    if(code[0] !== '#') {
      return false;
    }
    for(let i = 1; i < len; i++) {
      if(!((0 <= code[i] && code[i] <= 9) || ('A' <= code[i] && code[i] <= 'F'))) {
        return false;
      }
    }
    return true;
  }
  return false;
}

const generatePalette = ({ module: modules, color_map_preset: colorMapPreset, color_map_palette: colorMapPalette, preset_rows: presetRows, preset_cols: presetCols, palette_rows: paletteRows, palette_cols: paletteCols }) => {
  let html = '';

  if(modules.includes('preset')) {
    html += "<div class='colorPalette-div colorPalette-div-preset'><table class='colorPalette-preset-table'>"
    for(let i = 0; i < presetRows; i++) {
      html += "<tr class='colorPalette-preset-tr'>";
      for(let j = 0; j < presetCols; j++) {
        const curColor = colorMapPreset[i*presetCols+j];
        html += `<td class='colorPalette-preset-td' id=${'colorPalette-preset-id-'+(i*presetCols+j)} style='background:${curColor}' data-mce-color=${curColor}><span class='colorPalette-preset-check'></span></td>`
      }
      html += "</tr>"
    }
    html += "</table></div>"
  }

  if(modules.includes('palette')) {
    html += "<div class='colorPalette-div colorPalette-div-palette'><table class='colorPalette-palette-table'>"
    for(let i = 0; i < paletteRows; i++) {
      html += "<tr class='colorPalette-palette-tr'>"
      for(let j = 0; j < paletteCols; j++) {
        const curColor = colorMapPalette[i*paletteCols+j];
        html += `<td class='colorPalette-palette-td' id=${'colorPalette-palette-id-'+(i*14+j)} style='background:${curColor};' data-mce-color=${curColor}></td>`
      }
      html += "</tr>"
    }
    html += "</table></div>"
  }

  if(modules.includes('code')) {
    html += "<div class='colorPalette-div colorPalette-div-code'>"
    html += "<div id='colorPalette-code-display' class='colorPalette-code-display'></div><input id='colorPalette-code-input' class='colorPalette-code-input'></input><button id='colorPalette-code-submit' class='colorPalette-code-submit' disabled>선택</button></div>";
  }

  return html;
};


export default plugin;
