const NodeTypeNameObject = {
  headingArray: ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'],
  textArray: ['i', 'b', 'u', 'del','span', '#text'],
  image: 'img',
  paragraph: 'p',
  horizontalRule: 'hr',
  division: 'div',
  iframe: 'iframe',
  codeBlockWrap: 'pre',
  codeBlockCode: 'code'
}

class DataModel {
  constructor(tree) {
    this.rootTreeContent = tree
    this.contents = []
    this.images = []
    this.videos = []
    this.maps = []
  }

  getStyleObjectByAttirbute(styleString) {
    const styleArray = styleString.split(';');
    const resultOjbect = {};
    for (let styleValue of styleArray) {
      if (styleValue) {
        styleValue = styleValue.trim()
        let valuePArray = styleValue.split(':')
        let keyString = valuePArray[0].trim();
        let valueString = valuePArray[1].trim();
        keyString && valueString && (resultOjbect[keyString] = valueString)
      }
    }
    return resultOjbect
  }

  getStylePropertyValueInAttirbute(treeObject, styleProperty) {
    const attribute = treeObject.attributes
    if (attribute && attribute.map && attribute.map.style) {
      const styleObject = this.getStyleObjectByAttirbute(attribute.map.style)
      if (styleObject[styleProperty]) {
        return styleObject[styleProperty]
      } else {
        return null;
      }
    }
    return null;
  }

  setFontStyleAttribute(treeObject, itemAttributeObject) {
    const nodeName = treeObject.name;
    if (nodeName == 'strong') {
      itemAttributeObject.font_style = 'bold'
    } else if (nodeName == 'em') {
      itemAttributeObject.font_style = 'italic'
    } else if (nodeName == 'span') {
      const textDecorationValue = this.getStylePropertyValueInAttirbute(treeObject, 'text-decoration');
      if (textDecorationValue) {
        itemAttributeObject.font_style = textDecorationValue
      }
    }
  }

  setFontColorAttribute(treeObject, itemAttributeObject) {
    const attribute = treeObject.attributes
    if (attribute && attribute.map && attribute.map.style) {
      const styleObject = this.getStyleObjectByAttirbute(attribute.map.style)
      if (styleObject.color) {
        itemAttributeObject.font_color = styleObject.color
      }
    }
  }

  setTextAlignAttribute(treeObject, itemAttributeObject) {
    const attribute = treeObject.attributes
    if (attribute && attribute.map && attribute.map.style) {
      const styleObject = this.getStyleObjectByAttirbute(attribute.map.style)
      if (styleObject['text-align']) {
        itemAttributeObject.text_align = styleObject['text-align']
      }
    }
  }

  getKeType(obj) {
    if (obj && obj.attributes && obj.attributes.map) {
      return obj.attributes.map['data-ke-type']
    } else {
      return ''
    }
  }

  makeData(treeObject, parentArray) {
    let itemObject = {}, type, itemAttribute = {}, fontStyle, fontColor, url, headingLevel, stopChildPropagation = false
    const keType = this.getKeType(treeObject)

    if (treeObject.name == NodeTypeNameObject.paragraph) {
      type = 'paragraph'
      this.setTextAlignAttribute(treeObject, itemAttribute)
    } else if (NodeTypeNameObject.headingArray.indexOf(treeObject.name) > -1) {
      type = 'heading'
      headingLevel = NodeTypeNameObject.headingArray.indexOf(treeObject.name) + 2;
      this.setTextAlignAttribute(treeObject, itemAttribute)
      itemAttribute.heading_level = headingLevel
    } else if (treeObject.name == NodeTypeNameObject.horizontalRule) {
      type = 'horizontalRule'
    } else if (NodeTypeNameObject.textArray.indexOf(treeObject.name) > -1) {
      type = 'text'
      this.setFontStyleAttribute(treeObject, itemAttribute)
      this.setFontColorAttribute(treeObject, itemAttribute)
      if (treeObject.value) {
        itemAttribute.text = treeObject.value
      }
    } else if (treeObject.name == NodeTypeNameObject.image) {
      type = 'image'
      itemAttribute.url = treeObject.attributes.map.src
      itemAttribute.file_name = treeObject.attributes.map['data-filename']
      itemAttribute.width = treeObject.attributes.map.width
      itemAttribute.height = treeObject.attributes.map.height
      this.images.push(itemAttribute)
    } else if (treeObject.name == 'figcaption') {
      type = 'caption'
    } else if (treeObject.name == NodeTypeNameObject.codeBlockWrap) {
      if (treeObject.firstChild && treeObject.firstChild.name == NodeTypeNameObject.codeBlockCode) {
        type = 'codeBlock'
        itemAttribute.text = treeObject.firstChild.firstChild.value || ''
        stopChildPropagation = true
      }
    } else if (['video', 'map', 'image'].indexOf(keType) >= 0) {
      if (keType == 'image') {
        type = 'imageWrap'
      } else  if (keType == 'map') {
        type = 'map'
        itemAttribute.map_data = treeObject.attributes.map['data-maps-data']
        itemAttribute.id = treeObject.attributes.map.id
        stopChildPropagation = true
        this.maps.push(itemAttribute)
      } else  if (keType == 'video') {
        type = 'video'
        itemAttribute.url = treeObject.attributes.map['data-video-url']
        itemAttribute.thumbnail = treeObject.attributes.map['data-video-thumbnail']
        itemAttribute.vid = treeObject.attributes.map['data-video-vid']
        itemAttribute.uploaderHost = treeObject.attributes.map['data-video-uploader-host']
        this.videos.push(itemAttribute)
      }
    } else {
      type = 'else'
    }
    if (type) {
      if (Object.keys(itemAttribute).length != 0) {
        itemObject.attribute = itemAttribute;
      }
      itemObject.type = type
      parentArray.push(itemObject)
    }
    if (!stopChildPropagation && treeObject.firstChild) {
      if (!itemObject.child) {
        itemObject.child = []
      }
      this.makeData(treeObject.firstChild, itemObject.child)
    }
    if (treeObject.next) {
      this.makeData(treeObject.next, parentArray)
    }
  }

  getData() {
    this.makeData(this.rootTreeContent.firstChild, this.contents)

    return {
      contents: this.contents,
      images: this.images,
      videos: this.videos,
      maps: this.maps
    }
  }
}

export default DataModel