import relocateCursor from '../components/relocateCursor';
import copyCutHandler from '../components/copyCutHandler';
import regex from '../components/regex';
import isWordContent from '../components/isWordContent';
import htmlHandler from '../components/htmlHandler';
import imageHandler from '../components/imageHandler';
import unescapeHtml from '../components/unescapeHtml';
import bowser from 'bowser';
import rm from '../components/safeRemove';

const plugin = editor => {

  const delegatedDelete = (rng) => {
    return () => {
      editor.selection.setRng(rng);
      editor.undoManager.ignore(() => {
        editor.execCommand('Delete');
      })
      editor.nodeChanged();
    }
  }

  const preprocess = (itemsObject, originalRange) => {
    const sel = editor.selection.getSel();
    let rng;
    if(!sel.isCollapsed) {
      rng = sel.getRangeAt(0).cloneRange();
    }
    const targetNode = relocateCursor(editor);
    itemsObject.targetNode = targetNode;

    if(rng && !rng.collapsed) {
      if(rng.commonAncestorContainer.nodeName !== 'FIGURE') {
        itemsObject.deleteRange = delegatedDelete(rng);
      }
    }

    const boundHandlerDecision = handlerDecision.bind(itemsObject);
    isWordContent(itemsObject.htmlItem, boundHandlerDecision);
  }

  const processDecision = (items) => {
    let imageItem, htmlItem, plainTextItem;
    for(let i = 0; i < items.length; i++) {
      const currentItem = items[i];

      if(currentItem.type.indexOf('image') >= 0) {
        imageItem = currentItem;
      }
      else if(currentItem.type.indexOf('html') >= 0) {
        htmlItem = currentItem;
      }
      else {
        plainTextItem = currentItem;
      }
    }

    const itemsObject = {
      imageItem: imageItem? imageItem.getAsFile(): null,
      htmlItem,
      plainTextItem
    }
    return itemsObject;
  }

  const handlerDecision = function(isWord, wordHtml) {
    if(this.imageItem) {
      if(isWord) {
        htmlHandler(editor, wordHtml, this.targetNode, this.deleteRange);
      }
      else {
        if(this.targetNode === 'TABLE') {
          return;
        }
        if(this.deleteRange) this.deleteRange();
        imageHandler(editor, this.imageItem);
      }
    }
    else if(this.htmlItem) {
      htmlHandler(editor, wordHtml, this.targetNode, this.deleteRange);
    }
    else {
      if(this.plainTextItem) {
        if(this.plainTextItem.getAsString) {
          this.plainTextItem.getAsString(plainTextItem => {
            plainTextHandler(plainTextItem, this.deleteRange)
          });
        }
        else {
          plainTextHandler(this.plainTextItem, this.deleteRange);
        }
      }
    }
  }

  const plainTextHandler = (plainTextItem, deleteRange) => {
    if(typeof plainTextItem !== "string") {
      plainTextItem = plainTextItem.data;
    }
    const p = document.createElement('p');
    const elementList = plainTextItem.split(/(\n)/);
    const resultList = [];
    elementList.forEach(el => {
      if(el === '\n') {
        resultList.push(document.createElement('br'));
      }
      else {
        const innerElementList = el.split(regex.regexString.isUrl);
        innerElementList.forEach(innerEl => {
          if(regex.isUrlInText(innerEl)) {
            const a = document.createElement('a');
            a.setAttribute('href', innerEl);
            a.textContent = innerEl;
            resultList.push(a);
          }
          else {
            resultList.push(document.createTextNode(innerEl));
          }
        })
      }
    })
    if(deleteRange)
      deleteRange();

    resultList.forEach(el => {
      el.textContent = el.textContent.replace(/ /gi, '\u00A0');
      p.appendChild(el);
    })
    editor.insertContent(p.innerHTML);
    editor.nodeChanged();
    rm(p);
  }

  const kPasteInit = () => {

    if(bowser.name === 'Internet Explorer') {
      iePastebinInit();
      let bm;
      editor.on('beforepaste', e => {
        editor.undoManager.ignore(() => {
          const isCollapsed = editor.selection.getSel().isCollapsed;
          bm = editor.selection.getBookmark();
          const iePastebin = tinymce.$('#ie_pastebin');
          iePastebin[0].focus()
          const range = document.createRange();
          range.selectNodeContents(iePastebin[0]);
          const selection = window.getSelection();
          selection.removeAllRanges();
          selection.addRange(range);
        })
      });

      document.addEventListener('paste', () => {
        const iePastebin = tinymce.$('#ie_pastebin');
        const isCollapsed = editor.selection.getSel().isCollapsed;

        setTimeout(() => {
          const itemsObject = {}
          itemsObject.htmlItem = unescapeHtml(iePastebin.html());

          editor.focus();
          if(bm) {
            editor.selection.moveToBookmark(bm);
          }
          iePastebin.empty();
          preprocess(itemsObject);
        }, 0)
        return true;
      })

      editor.addCommand('ieCut', (e) => {
        editor.undoManager.transact(() => copyCutHandler(editor, e, false));
      })

      editor.addCommand('ieCopy', (e) => {
        editor.undoManager.transact(() => copyCutHandler(editor, e, true));
      })

      editor.on('keyDown', e => {
        if(e.ctrlKey) {
          if(e.keyCode === 67) {
            e.preventDefault();
            editor.undoManager.ignore(() => {
              editor.execCommand('ieCopy', e);
            })
          }
          else if(e.keyCode === 88) {
            e.preventDefault();
            editor.undoManager.ignore(() => {
              editor.execCommand('ieCut', e);
            })
          }
        }
      }, true);
    }

    else { // chrome, safari, firefox
      editor.on('paste', e => {
        e.preventDefault();
        let itemsObject;
        if(e.clipboardData.items) { // chrome & firefox
          itemsObject = processDecision(e.clipboardData.items);
        }
        else { // safari
          itemsObject = {
            imageItem: undefined,
            htmlItem: e.clipboardData.getData('text/html'),
            plainTextItem: e.clipboardData.getData('text/plain'),
          }
        }
        preprocess(itemsObject);
      })

      editor.on('copy', e => {
        e.preventDefault();
        copyCutHandler(editor, e, true);
      }, true)

      editor.on('cut', e => {
        e.preventDefault();
        editor.undoManager.transact(() => copyCutHandler(editor, e, false));
      }, true)
    }
  }

  kPasteInit();
}

const iePastebinInit = () => {
  const pastebin = document.createElement('div');
  document.body.appendChild(pastebin);
  pastebin.id = 'ie_pastebin';
  pastebin.setAttribute('style', 'width: 0px; height: 0px; overflow: hidden;');
  pastebin.setAttribute('contenteditable', true);
}

// remove() polyfill
(function (arr) {
  arr.forEach(function (item) {
    if (item.hasOwnProperty('remove')) {
      return;
    }
    Object.defineProperty(item, 'remove', {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function remove() {
        if (this.parentNode !== null)
          this.parentNode.removeChild(this);
      }
    });
  });
})([Element.prototype, CharacterData.prototype, DocumentType.prototype]);

export default plugin;
