import '../style/location.scss'
import autobind from 'autobind-decorator'

class App {
  constructor(editor) {
    this.editor = editor;
    this.win = null;
    this.inputTimeout = undefined;
    this.loadTimeout = undefined;
    this.settings = editor.settings.location;
    this.fetchHandler = this.settings.location && this.settings.location.fetch_handler ? this.settings.location.fetch_handler : () => {}
  }

  open() {
    let win = this.editor.windowManager.open({
      title: '장소정보 삽입',
      items: [{
        type: 'container',
        html: '<div class="mce-location" style="width:600px; height:534px">\
          <div class="mce-location-header">\
            <input class="mce-location-input" placeholder="장소명을 입력해주세요.">\
            <button class="mce-location-search"><i class="mce-ico mce-plus">확인</i></button>\
          </div>\
          <div class="mce-location-body"></div>\
          <div class="mce-location-footer">\
            <button class="mce-location-cancel">취소</button>\
          </div>\
        </div>'
      }],
      buttons: []
    });
    win.statusbar.remove()

    this.$input = win.$('.mce-location-input')
    this.$btnSearch = win.$('.mce-location-search')
    this.$body = win.$('.mce-location-body')
    this.$btnCancel = win.$('.mce-location-cancel')
    this.$window = win.$(window)
    this.$modalBlock = win.$('#mce-modal-block')

    this.$input.on("keydown", this.onKeydown)
    this.$input.on("input", this.onInput)
    this.$btnSearch.on("click", this.onSearch)
    this.$btnCancel.on("click", this.onCancel)
    this.$window.on("keydown", this.onWindowKeydown)
    this.$modalBlock.on("click", this.onCancel)
    this.$input[0].focus()

    this.win = win

    this.setText('INITIAL');
  }

  close() {
    if (this.win) {
      this.$input.off("keydown")
      this.$btnSearch.off("click")
      this.$btnCancel.off("click")
      this.$window.off("keydown")
      this.$modalBlock.off("click")
      this.clearInputTimeout();
      this.clearLoadTimeout();

      this.win.close()

      this.$input = null
      this.$body = null
      this.win = null
    }
  }

  generatingListItems(data, page, isEnd) {
    let len = data.length;
    let ret = ''
    for(let i = 0; i < len; i++) {
      ret += `<li class="mce-location-result-place" mce-data-location-url=${data[i]["place_url"]}\
       mce-data-location-coordinateX=${data[i]["x"]} mce-data-location-coordinateY=${data[i]["y"]}\
        mce-data-location-place-id=${data[i]["id"]}><div class="mce-location-result-place-content">\
        ${data[i]["place_name"]}<em class="mce-location-result-address">${data[i]["road_address_name"]}</em></div></li>`
    }
    if(!isEnd) {
      ret += `<li class="mce-location-result-more" mce-data-location-page=${page}>Load More...</li>`;
    }
    return ret;
  }

  renderItems(data, page, isEnd) {
    if(page === 1) {
      let html = '<ul class="mce-location-place-list">' + this.generatingListItems(data, page, isEnd) + '</ul>';
      this.$body.html(html);
      this.$body.on("scroll", this.onScroll);
    }
    else { // page > 1
      this.$loadMore.remove();
      let html = this.generatingListItems(data, page, isEnd);
      this.$body[0].children[0].insertAdjacentHTML('beforeend', html);
    }

    this.win.$('.mce-location-result-place').on("click", this.onListItemClicked);

    if(!isEnd) {
      this.$loadMore = this.win.$('.mce-location-result-more');
      this.$loadMore.on("click", this.onScrollToEnd);
    }
    else {
      this.$body.off("scroll");
    }
  }

  loadMore(query, page) {
    this.fetchHandler(query, page, (data) => {
      this.renderItems(data.documents, page, data.meta.is_end);
    })
  }

  @autobind
  onKeydown(e) {
    let key = e.keyCode;
    if(key == 9 || key == 13) {
      this.onSearch();
    }
  }

  @autobind
  onSearch() {
    if(this.$loadMore) {
      this.$loadMore.off("click");
      this.$loadMore = null;
    }

    let value = this.$input[0].value
    if(value) {
      this.fetchHandler(value, 1, (data) => {
        if(data.documents.length > 0) {
          this.renderItems(data.documents, 1, data.meta.is_end);
        }
        else {
          this.setText('NO_RESULT');
        }
      })
    }
  }

  @autobind
  onCancel(e) {
    this.close();
  }

  @autobind
  onWindowKeydown(e) {
    if(e.keyCode == 27) {
      this.close();
    }
  }

  @autobind
  onScrollToEnd(e) {
    this.$loadMore.html("Loading...");
    this.$loadMore.className = 'mce-location-result-more-disabled';
    let value = this.$input[0].value;
    this.loadMore(value, Number(this.$loadMore[0].attributes['mce-data-location-page'].value) + 1);
  }

  @autobind
  onListItemClicked(e) {
    const place = e.currentTarget;
    let placeName = place.childNodes[0].childNodes[0];
    let placeUrl = undefined;
    if(this.editor.settings.location.link_to_map) {
      placeUrl = `http://map.daum.net/?latlng=${place.attributes['mce-data-location-coordinateX'].value},\
      ${place.attributes['mce-data-location-coordinateX'].value}\
      &q=${placeName.textContent}&itemId=${place.attributes['mce-data-location-place-id'].value}`
    }
    else {
      placeUrl = place.attributes['mce-data-location-url'].value
    }

    const locationWrapper = this.editor.dom.create('p');
    const locationString = this.editor.dom.create('a', { href: placeUrl }, placeName);
    this.editor.dom.add(locationWrapper, locationString);
    this.editor.insertContent(locationWrapper.outerHTML);
    this.close();
  }

  @autobind
  onScroll(e) {
    if(this.$body[0].offsetHeight + this.$body[0].scrollTop === this.$body[0].scrollHeight && this.$loadMore) {
      this.clearLoadTimeout();
      this.loadTimeout = setTimeout(() => {
        this.onScrollToEnd()
      }, 50);
    }
  }

  @autobind
  onInput(e) {
    this.clearInputTimeout();
    const query = this.$input[0].value;
    if(query === '') {
      this.setText('INITIAL');
    }
    else {
      this.inputTimeout = setTimeout(() => {
        this.onSearch()
      }, 500);
    }
  }

  setText(type) {
    if(type === 'NO_RESULT') {
      this.$body.html('<p class="mce-location-text">검색 결과가 없습니다.</p>')
    }
    else if(type === 'INITIAL') {
      this.$body.html('<p class="mce-location-text">검색어를 입력해주세요.</p>')
    }
  }

  clearInputTimeout() {
    if(this.inputTimeout) {
      clearTimeout(this.inputTimeout);
    }
  }

  clearLoadTimeout() {
    if(this.loadTimeout) {
      clearTimeout(this.loadTimeout);
    }
  }
}

export default App;
